//---------------------------------------------------------------------------
// Copyright (C) 2001 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//     Module Name: SamLed
//
//     Description: DS2406 LED functions using 'Sam' type low level functions
//
//        Filename: SamLed.c
//
// Dependant Files: jibapi.h, Sam32 - low level functions
//
//          Author: DS
//
//        Compiler: MSVC on Win32 x86 ONLY
//
//         Version: Ver 0.00
//
//         Created: 03/06/01
//   

// includes
#include "jibapi.h"

// external low-level functions 
extern uchar iBOverdriveOn(void);
extern void iBOverdriveOff(void);
extern uchar iBDOWCheck(void);
extern uchar iBKeyOpen(void);
extern uchar iBKeyClose(void);
extern uchar iBFirst(void);
extern uchar iBNext(void);
extern uchar iBFastAccess(void);
extern uchar iBStrongAccess(void);
extern uchar iBAccess(void);
extern uchar *iBROMData(void);
extern uchar iBDataByte(uchar data);
extern uchar iBDataBlock(uchar *data, int count);
extern uchar iBSetup(uchar pn);
extern uchar iBDataBit(uchar bit);
extern uchar iBDOWReset();
extern uchar SetAdapterType(uchar Type, char *type);
extern uchar SetAdapterSpeed(ulong speed);
extern uchar SetAdapter5VTime(uchar time);
extern uchar Adapter5VPrime(void);
extern uchar Adapter5VCancel(void);
extern int Adapter5VActivate(ulong ustimeout);
extern uchar iBStreamRS(uchar *arr,int length,int run);
extern uchar iBStream(uchar *arr,int length);

// local exportable prototypes
BOOL DS2406LEDInit();
BOOL DS2406LEDOn();
BOOL DS2406LEDOff();

// local static prototypes
static BOOL SetDS2406FlipFlop(uchar *SwitchAddress, uchar ffstate);
static ushort crc16(int data);

// local statics
static uchar SwitchAddress[8];
static ushort CRC16;

// defs
#define AOFF_BOFF       0x73
#define AON_BOFF        0x53

//-----------------------------------------------------------------------
// Description: Find and Initialize DS2406 that controls an LED in the
//              Main channel.
//
// NOTE: This functions assumes that iBDOWCheck(), SetAdapterType() and 
//       iBSetup() has already been called.  If Using the JibAPI, then 
//       doing any operation (like  GetFirmwareVersionString()) will 
//       properly call all of these functions.
//
// Returns:     FALSE on failure, TRUE on success
//
BOOL DS2406LEDInit()
{
   uchar *rom_ptr, keep_rom[8], search_rslt;
   BOOL rt=FALSE;

   // get control of the 1-Wire
   if (iBKeyOpen())
   {
      // remember what is the current ROM
      rom_ptr = iBROMData();
      memcpy(keep_rom,rom_ptr,8);

      // search for the DS2406
      iBOverdriveOff();
      search_rslt = iBFirst();
      while (search_rslt)
      {
         if ((rom_ptr[0] & 0x7F) == 0x12)
         {
            memcpy(SwitchAddress,rom_ptr,8);
            rt = SetDS2406FlipFlop(SwitchAddress, AOFF_BOFF);
            break;
         }

         search_rslt = iBNext();
      }

      // restore the current ROM
      memcpy(rom_ptr,keep_rom,8);

      // release the 1-Wire
      iBKeyClose();
   }

   return rt;
}

//-----------------------------------------------------------------------
// Description: Turn On DS2406 LED
//
// Returns:     FALSE on failure, TRUE on success
//
BOOL DS2406LEDOn()
{
   BOOL rslt = FALSE;

   // get control of the 1-Wire
   if (iBKeyOpen())
   {
      iBOverdriveOff();
      rslt = SetDS2406FlipFlop(SwitchAddress, AON_BOFF);
      
      // release the 1-Wire
      iBKeyClose();
   }      

   return rslt;
}

//-----------------------------------------------------------------------
// Description: Turn Off DS2406 LED
//
// Returns:     FALSE on failure, TRUE on success
//
BOOL DS2406LEDOff()
{
   BOOL rslt = FALSE;

   // get control of the 1-Wire
   if (iBKeyOpen())
   {
      iBOverdriveOff();
      rslt = SetDS2406FlipFlop(SwitchAddress, AOFF_BOFF);
      
      // release the 1-Wire
      iBKeyClose();
   }      

   return rslt;
}

//----------------------------------------------------------------------
// Set the DS2406 flip flops to the 'ffstate' .
//                                                          
BOOL SetDS2406FlipFlop(uchar *SwitchAddress, uchar ffstate)
{    
   short repeat=0;
   uchar send_block[30];
   short send_cnt=0;
   uchar *rom_ptr, keep_rom[8];
   BOOL rt=FALSE; 

   // remember what is the current ROM
   rom_ptr = iBROMData();
   memcpy(keep_rom,rom_ptr,8);

   // set the ROM number
   memcpy(rom_ptr,SwitchAddress,8);

   do
   {
      if (iBStrongAccess())
      {   
         // create a block to send the sets the state
         send_cnt = 0; 
         CRC16 = 0;

         // write status command
         send_block[send_cnt++] = 0x55;
         crc16(0x55);
         // address of switch state
         send_block[send_cnt++] = 0x07;
         crc16(0x07);
         send_block[send_cnt++] = 0x00;
         crc16(0x00);
         // write state
         send_block[send_cnt++] = (uchar)ffstate;
         crc16(ffstate);
         // read CRC16
         send_block[send_cnt++] = 0xFF;
         send_block[send_cnt++] = 0xFF;

         // now send the block
         if (iBDataBlock(send_block,send_cnt))
         {
            // perform the CRC16 on the last 2 bytes of packet
            crc16(send_block[send_cnt-2]);
            crc16(send_block[send_cnt-1]);

            // verify CRC16 is correct
            if (CRC16 == 0xB001)
            {
               // success
               rt = TRUE;
               break;
            }
         }
      }

      // delay to let things settle
      Sleep(10);
   }
   while (repeat++ < 10);

   // restore the current ROM
   memcpy(rom_ptr,keep_rom,8);

   return rt;       
}

//--------------------------------------------------------------------------
// Calculate a new CRC16 from the input data integer.  Return the current
// CRC16 and also update the global variable CRC16.
//
// 'data' - data to perform a CRC16 on
//
// Returns: the current CRC16
//
static short oddparity[16] = { 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 1, 1, 0 };

ushort crc16(int data)
{
   data = (data ^ (CRC16 & 0xff)) & 0xff;
   CRC16 >>= 8;

   if (oddparity[data & 0xf] ^ oddparity[data >> 4])
      CRC16 ^= 0xc001;

   data <<= 6;
   CRC16   ^= data;
   data <<= 1;
   CRC16   ^= data;

   return CRC16;
}
